package statechart

import scala.util.parsing.combinator.syntactical.StandardTokenParsers

/**
 * Statechart combinator parser without conversions.
 */
object SimpleStatechartParser extends StandardTokenParsers {

  def statechart = "statechart" ~> ident ~ ":" ~ properties ~ initialState ~ states
  
  def state = "state" ~> ident ~ ":" ~ opt(onEntry) ~ opt(onExit) ~ transitions
  
  def extendedState = "extendedState" ~> ident ~ ":" ~ substates ~ initialState ~ transitions
  
  def transition = ident ~ opt(guard) ~ "->" ~ ident
  
  def properties = "properties" ~> ":" ~> repsep(property, ",")
  
  def property = ident ~ "=" ~ initialValue
  
  def substates = "substates" ~> ":" ~> repsep(ident, ",")
  
  def initialValue = numericLit | "true" | "false"
  
  def transitions = "transitions" ~> ":" ~> rep(transition)  
  
  def guard = "[" ~> condition <~ "]"
  
  def condition = opt("not") ~ ident ~ opt(comparison)
  
  def comparison = symbol ~ numericLit
  
  def symbol = "gt" | "lt" | "eq"
  
  def states = rep1(state | extendedState)  
  
  def initialState = "initialState" ~> ":" ~> ident
  
  def onEntry = "onEntry" ~> ":" ~> repsep(ident, ",")
  
  def onExit = "onExit" ~> ":" ~> repsep(ident, ",")
  
  // Fuer den Lexer notwendig
  lexical.delimiters ++= List("[", "]", ",", "(", ")", ":", "=", "->")
  
  lexical.reserved ++= List("statechart", "properties", "extendedState",
      "substates", "transitions", "onEntry", "onExit", "not", "gt", "lt",
      "eq", "state", "initialState", "true", "false")  
}
